#! /usr/bin/env bash


main() {
    if [[ ! -e /tmp/wikipedia.html ]]; then
        curl 'https://en.wikipedia.org/wiki/Passengers_of_the_RMS_Titanic' > /tmp/wikipedia.html
    fi
    cat /tmp/wikipedia.html | perl -nw -e '
        BEGIN {
            $class=undef;
            %countries=( "US"=>"US", "UK"=>"UK", "Canada"=>"Canada", "France"=>"France", "Sweden"=>"Sweden", "Belgium"=>"Belgium", "Argentina"=>"Argentina", "New York City"=>"US", "England"=>"England", "Italy"=>"Italy", "Ireland"=>"Ireland", "Russia"=>"Russia", "Russian Empire"=>"Russia", "Spain"=>"Spain", "Switzerland"=>"Switzerland", "Egypt"=>"Egypt", "Peru"=>"Peru", "Mexico"=>"Mexico", "The Netherlands"=>"The Netherlands", "Scotland"=>"Scotland", "Cuba"=>"Cuba", "Uruguay"=>"Uruguay", "Portugal"=>"Portugal", "Siam"=>"Siam", "Japan"=>"Japan", "Denmark"=>"Denmark", "Monaco"=>"Monaco", "Norway"=>"Norway", "Channel Islands"=>"Channel Islands", "Germany"=>"Germany", "German Empire"=>"Germany", "Ottoman Empire"=>"Lebanon", "Australia"=>"Australia", "British India"=>"UK", "Lithuania"=>"Lithuania" );
        }
        {
                chomp;
                if ($_ =~ m/id="First_class_2"/) {
                    $class=1;
                    next;
                }
                if ($_ =~ m/id="Second_class_2"/) {
                    $class=2;
                    next;
                }
                if ($_ =~ m/id="Third_class_2"/) {
                    $class=3;
                    next;
                }
                if ($_ =~ m{</table>}) {
                    $class=undef;
                }
                defined $class or next;
                if ($_ =~ m{<tr style="background:#9bddff;">}) {
                    $survived=1;
                    $col=0;
                    next;
                }
                if ($_ =~ m{<tr>}) {
                    $survived=0;
                    $col=0;
                    next;
                }
                if ($_ =~ m{</tr>}) {
                    $col=undef;
                    $name=undef;
                    $age=undef;
                    $survived=undef;
                    $hometown=undef;
                    $homecountry=undef;
                    $boarded=undef;
                    $dest=undef;
                    $lifeboat=undef;
                    $title=undef;
                    $sex=undef;
                    $printit=0;
                    next;
                }
                defined $col or next;
                if ($_ =~ m{<td>(.*)</td>}) {
                    my $value=$1;
                    $value=~s/<[^>]+>//g;
                    $value=~s/&#91;/[/g;
                    $value=~s/&#93;/]/g;
                    if ($col==0) {
                        $name=$value;
                        if ($name =~ m/ (Mr\.|Master|Father|Reverend|Colonel|Major|Captain) /) {
                            $sex="M";
                            $title=$1;
                            $title=~s/\.//g;
                        } elsif ($name =~ m/ (Lady|Countess|Mrs\.|Ms\.|Miss) /) {
                            $sex="F";
                            $title=$1;
                            $title=~s/\.//g;
                        } elsif ($name =~ m/ Dr. /) {
                            $title="Dr";
                            if ($name =~ m/née/) {
                                $sex="F";
                            } else {
                                $sex="M";
                            }
                        } elsif ($name =~ m/ (Mr|Sir) /) {
                            $sex="M";
                            $title=$1;
                        } else {
                            if ($name =~ /née/) {
                                $sex="F";
                                $title="Mrs";
                            } elsif ($name =~ /maid/) {
                                $sex="F";
                                $title="Mrs";
                            } else {
                                # We verified that unspecified titles are all males.
                                $sex="M";
                                $title="Mr";
                            }
                        }
                    } elsif ($col==1) {
                        $age=$value;
                    } elsif ($col==2) {
                        $hometown=$value;
                    } elsif ($col==3) {
                        if ($class==3) {
                            $homecountry=$value;
                        } else {
                            $boarded=$value;
                        }
                    } elsif ($col==4) {
                        if ($class==3) {
                            $boarded=$value;
                        } else {
                            $dest=$value;
                        }
                    } elsif ($col==5) {
                        if ($class==3) {
                            $dest=$value;
                        } else {
                            $lifeboat=$value;
                            $printit=1;
                        }
                    } elsif ($col==6 && $class==3) {
                        $lifeboat=$value;
                        $printit=1;
                    }
                    if ($printit) {
                        if (!defined $homecountry) {
                            my $home=$hometown;
                            $home=~s/\s*\[.*\]$//;
                            if (exists $countries{$home}) {
                                $homecountry = $countries{$home};
                            } else {
                                my $word = ($home =~ /, ([^,]+)$/)[0];
                                if (defined $word && exists $countries{$word}) {
                                    $homecountry=$countries{$word};
                                } else {
                                    $home =~ /England/ and $homecountry='England';
                                }
                            }
                        }
                        print $name.";".$title.";".$sex.";".$age.";".$class.";".$survived.";".$hometown.";".$homecountry.";".$boarded.";".$dest."\n";
                        $printit=0;
                    }
                    $col++;
                }
        }
    ' > passengers.csv
}

main "$*"
